package es.unican.docencia.y2011.eda.practica2.objetos;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

/**
 * Tipo enumerado para representar las notas musicales. La clase
 * hace uso de las facilidades proporcionadas por el lenguaje Java 
 * para la manipulacin de tipos enumerados 
 * 
 * @author Pablo Sanchez (p.sanchez@unican.es)
 * 		   http://personales.unican.es/sanchezbp	
 *
 * ADVERTENCIA: Se han suprimido todos los acentos en los comentarios
 *   para evitar problemas de compatibilidad entre diferentes 
 *   computadoras 
 */
public enum NotaMusical {
	//--------------------------------------------------------//
	// Declaracion de los valores constantes para el tipo     //
	//--------------------------------------------------------//	
	Do, Re, Mi, Fa, Sol, La, Si;
	
	protected final static int MAX_NOTA_MUSICAL = NotaMusical.values().length;
	
	//-----------------------------------------------------------//
	// Funciones para manipular el tipo                          //
	//-----------------------------------------------------------//
	
	/**
	 * Constructor del valor literal Do
	 * @return El valor Do como NotaMusical
	 */
	static public NotaMusical Do() {
		return Do;
	} //  Do
	
	/**
	 * Constructor del valor literal Re
	 * @return El valor Re como NotaMusical
	 */
	static public NotaMusical Re() {
		return Re;
	} // Re

	/**
	 * Constructor del valor literal Mi
	 * @return El valor Mi como NotaMusical
	 */
	static public NotaMusical Mi() {
		return Mi;
	} // Mi

	/**
	 * Constructor del valor literal Fa
	 * @return El valor fA como NotaMusical
	 */
	static public NotaMusical Fa() {
		return Fa;
	} // Fa

	/**
	 * Constructor del valor literal Sol
	 * @return El valor Sol como NotaMusical
	 */	
	static public NotaMusical Sol() {
		return Sol;
	} // Sol
	
	/**
	 * Constructor del valor literal La
	 * @return El valor La como NotaMusical
	 */	
	static public NotaMusical La() {
		return La;
	} //  Do

	/**
	 * Constructor del valor literal Si
	 * @return El valor Si como NotaMusical
	 */	
	static public NotaMusical Si() {
		return Si;
	} //  Do
	
	//-----------------------------------------------------------//
	// Funciones para manipular el tipo                          //
	//-----------------------------------------------------------// 
	
	/**
	 * Devuelve la siguiente nota musical, de forma circular 
	 * @return La siguiente nota musical
	 */
	public NotaMusical siguiente() {
		return NotaMusical.values()[(this.ordinal() + 1) % NotaMusical.MAX_NOTA_MUSICAL];
	} // siguiente
	
	/**
	 * Devuelve la nota musical anterior a la que recibe la llamada, de forma circular 
	 * @return La nota musical anterior
	 */
	public NotaMusical anterior() {
		return NotaMusical.values()[(this.ordinal() - 1 + NotaMusical.MAX_NOTA_MUSICAL) 
		                            						% NotaMusical.MAX_NOTA_MUSICAL];
	} // anterior
	
	//-----------------------------------------------------------//
	// Funciones de entrada salida								 //
	//-----------------------------------------------------------//
	
	/**
	 * Lee por la consola el valor de un dia de la semana introducido como una cadena de caracteres
	 * Los valores vlidos son Do, Re, Mi, Fa, Sol, La y Si. 
	 * @param mensaje Mensaje que es mostrado por pantalla antes de pedir el valor de la nota musical  
	 * @return Nota Musical leida por teclado
	 * @throws IOException Fallo en la entrada estandar
	 */
	public static NotaMusical LeeNota(String mensaje) throws IOException {
		
		NotaMusical d = Do;
		
		BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
		
		boolean ok;
		
		do {
			ok = true;
			System.out.print(mensaje);
			String s = br.readLine();
			try {
				d = NotaMusical.valueOf(s);
			} catch (IllegalArgumentException e) {
				ok = false;
				System.out.println("Valor introducido incorrecto, introduzcalo de nuevo.");
			} // try-catch
		} while (!ok);	
		
		return d;
		
	} // LeerDiaSemana
	
	/**
	 * Pasa la nota musical a una cadena de caracteres en el sisteama latino 
	 * Se ha suprimido el primer parametro con respecto a la especificacin de la 
	 * prctica al entenderse que ese primer parametro es el objeto que recibe la llamada   
	 */
	public String aCadena() {
		return this.toString();
	} // ImprimeNota
	
	/**
	 * Imprime la nota musical actual por pantalla en notacin inglesa 
	 * Se ha suprimido el primer parametro con respecto a la especificacin de la 
	 * prctica al entenderse que ese primer parametro es el objeto que recibe la llamada   
	 */
	public String aCadenaInglesa() {
		
		String result = null;
		
		switch(this) {
		case Do:
			result = "C";
			break;
		case Re:
			result = "D";
			break;
		case Mi:
			result = "E";;
			break;
		case Fa:
			result = "F";;
			break;
		case Sol:
			result = "G";;
			break;
		case La:
			result = "A";;
			break;
		case Si:
			result = "B";;
			break;
		// Salvo que se introduzcan nuevas notas musicales, 
		// nunca deberia darse este caso. Se introduce por 
		// seguridad	
		default:
			throw new IllegalArgumentException();
		} // switch
		
		return result;
	} // ImprimeNota
	
} // NotaMusical
